-----------------------------------
-- Setting up scope and libs
-----------------------------------

local AddonName = "iGuild"
iGuild = LibStub("AceAddon-3.0"):NewAddon(AddonName, "AceEvent-3.0", "AceTimer-3.0");

local QTip = LibStub("LibQTip-1.0");
local Tourist = LibStub("LibTourist-3.0");
local L = LibStub("AceLocale-3.0"):GetLocale(AddonName);

-----------------------------------------
-- Variables, functions and colors
-----------------------------------------

local Tooltip;

local Roster = {};
local RosterTimer;

local ROSTER_CHAR_NAME, ROSTER_CHAR_LEVEL, ROSTER_CHAR_CLASS, ROSTER_CHAR_ZONE, ROSTER_CHAR_STATUS, ROSTER_CHAR_MOBILE = 1,2,3,4,5,6;
local ROSTER_ACM_POINTS, ROSTER_ACM_RANK, ROSTER_GUILD_RANK, ROSTER_GUILD_NOTE, ROSTER_OFFICER_NOTE = 7,8,9,10,11;

local NULL = "";

local COLOR_GOLD = "|cfffed100";
local COLOR_BNET = FRIENDS_BNET_NAME_COLOR_CODE;
local COLOR_OFFICER = "|cff40c040";

local ClassColors = {};
for k, v in pairs(LOCALIZED_CLASS_NAMES_MALE) do
	local c = RAID_CLASS_COLORS[k];
	ClassColors[v] = ("|cff%02x%02x%02x"):format(c.r *255, c.g *255, c.b *255);
end
for k, v in pairs(LOCALIZED_CLASS_NAMES_FEMALE) do
	local c = RAID_CLASS_COLORS[k];
	ClassColors[v] = ("|cff%02x%02x%02x"):format(c.r *255, c.g *255, c.b *255);
end

local function tclear(t, wipe)
	if( type(t) ~= "table" ) then return end;
	for k in pairs(t) do
		t[k] = nil;
	end
	t[''] = 1;
	t[''] = nil;
	if( wipe ) then
		t = nil;
	end
end

-----------------------------
-- Setting up the feed
-----------------------------

local function Feed_OnClick(_, button)
	if( button == "LeftButton" ) then
		if( IsAltKeyDown() and CanGuildInvite() ) then
			if( StaticPopup_FindVisible("ADD_GUILDMEMBER") ) then
				StaticPopup_Hide("ADD_GUILDMEMBER");
			else
				StaticPopup_Show("ADD_GUILDMEMBER");
			end
		else
			ToggleGuildFrame(1);
		end
	elseif( button == "RightButton" ) then
		iGuild:OpenOptions();
	end
end

local function Feed_OnEnter(anchor)
	if( not IsInGuild() ) then
		return;
	end
	
	for k, v in QTip:IterateTooltips() do
		if( type(k) == "string" and strsub(k, 1, 6) == "iSuite" ) then
			v:Release(k);
		end
	end
		
	Tooltip = QTip:Acquire("iSuite"..AddonName);
	Tooltip:SetAutoHideDelay(0.1, anchor);
	Tooltip:SmartAnchorTo(anchor);
	iGuild:UpdateTooltip();
	Tooltip:Show();
end

iGuild.Feed = LibStub("LibDataBroker-1.1"):NewDataObject(AddonName, {
	type = "data source",
	text = "",
	icon = "Interface\\Addons\\iGuild\\Images\\iGuild",
	OnClick = Feed_OnClick,
	OnEnter = Feed_OnEnter,
});

-----------------------------
-- Sorting and Columns
-----------------------------

iGuild.Sort = {
	name = function(a, b)
		return a[ROSTER_CHAR_NAME] < b[ROSTER_CHAR_NAME];
	end,
	level = function(a, b)
		if( a[ROSTER_CHAR_LEVEL] < b[ROSTER_CHAR_LEVEL] ) then
			return true;
		elseif( a[ROSTER_CHAR_LEVEL] > b[ROSTER_CHAR_LEVEL] ) then
			return false;
		else
			return iGuild.Sort.name(a, b);
		end
	end,
	class = function(a, b)
		if( a[ROSTER_CHAR_CLASS] < b[ROSTER_CHAR_CLASS] ) then
			return true;
		elseif( a[ROSTER_CHAR_CLASS] > b[ROSTER_CHAR_CLASS] ) then
			return false;
		else
			return iGuild.Sort.name(a, b);
		end
	end,
	rank = function(a, b)
		if( a[ROSTER_GUILD_RANK] < b[ROSTER_GUILD_RANK] ) then
			return true;
	elseif( a[ROSTER_GUILD_RANK] > b[ROSTER_GUILD_RANK] ) then
			return false;
		else
			return iGuild.Sort.name(a, b);
		end
	end,
	points = function(a, b)
		if( a[ROSTER_ACM_POINTS] > b[ROSTER_ACM_POINTS] ) then
			return true;
		elseif( a[ROSTER_ACM_POINTS] < b[ROSTER_ACM_POINTS] ) then
			return false;
		else
			return iGuild.Sort.name(a, b);
		end
	end
};

iGuild.Columns = {
	level = {
		Label = L["Level"],
		Brush = function(v)
			local c = GetQuestDifficultyColor(v[ROSTER_CHAR_LEVEL]);
			return ("|cff%02x%02x%02x%s|r"):format(c.r *255, c.g *255, c.b *255, v[ROSTER_CHAR_LEVEL]);
		end,
	},
	name = {
		Label = L["Name"],
		Brush = function(v)
			return ("%s%s%s|r"):format(ClassColors[v[ROSTER_CHAR_CLASS]], v[ROSTER_CHAR_STATUS] or NULL, v[ROSTER_CHAR_NAME]);
		end,
	},
	zone = {
		Label = L["Zone"],
		Brush = function(v)
			local r, g, b = Tourist:GetFactionColor(v[ROSTER_CHAR_ZONE]);
			return ("|cff%02x%02x%02x%s|r"):format(r *255, g *255, b *255, v[ROSTER_CHAR_ZONE]);
		end,
	},
	rank = {
		Label = L["Rank"],
		Brush = function(v)
			return ("%s%s|r"):format(COLOR_GOLD, v[ROSTER_GUILD_RANK]);
		end,
		Script = function(_, v, button)
			if( button == "LeftButton" and CanGuildPromote() ) then
				GuildPromote(v[ROSTER_CHAR_NAME]);
			end
			if( button == "RightButton" and CanGuildDemote() ) then
				GuildDemote(v[ROSTER_CHAR_NAME]);
			end
		end,
		ScriptUse = function() return ( CanGuildPromote() or CanGuildDemote() ) end,
	},
	note = {
		Label = L["Note"],
		Brush = function(v)
			return ("%s%s|r"):format(COLOR_GOLD, v[ROSTER_GUILD_NOTE]);
		end,
	},
	officernote = {
		Label = L["OfficerNote"],
		Brush = function(v)
			return ("%s%s|r"):format(COLOR_OFFICER, v[ROSTER_OFFICER_NOTE]);
		end,
		CanUse = function() return CanViewOfficerNote() end,
	},
	notecombi = {
		Label = L["Note"],
		Brush = function(v)
			local normal = false;
			local officer = false;
			
			if( v[ROSTER_GUILD_NOTE] and v[ROSTER_GUILD_NOTE] ~= "" ) then
				normal = true;
			end
			if( CanViewOfficerNote() and v[ROSTER_OFFICER_NOTE] and v[ROSTER_OFFICER_NOTE] ~= "" ) then
				officer = true;
			end
			
			local note = NULL;
			
			if( normal and not officer ) then
				note = ("%s%s|r"):format(COLOR_GOLD, v[ROSTER_GUILD_NOTE]);
			elseif( officer and not normal ) then
				note = ("%s%s|r"):format(COLOR_OFFICER, v[ROSTER_OFFICER_NOTE]);
			elseif( normal and officer ) then
				note = ("%s%s|r / %s%s|r"):format(COLOR_GOLD, v[ROSTER_GUILD_NOTE], COLOR_OFFICER, v[ROSTER_OFFICER_NOTE]);
			end
			
			return note;
		end,
	},
	acmpoints = {
		Label = L["Points"],
		Brush = function(v)
			return ("%s%s|r"):format(COLOR_GOLD, v[ROSTER_ACM_POINTS]);
		end,
	},
};

local DisplayedColumns = {};
function iGuild:GetDisplayedColumns()
	tclear(DisplayedColumns);
	
	for i = 1, #self.db.Display do
		local insert = true;
		local canUse = self.Columns[self.db.Display[i]].CanUse;
		
		if( type(canUse) == "boolean" and not canUse ) then
			insert = false;
		elseif( type(canUse) == "function" and not canUse() ) then
			insert = false;
		end
		
		if( insert ) then
			table.insert(DisplayedColumns, self.db.Display[i]);
		end
	end
	
end

----------------------
-- OnInitialize
----------------------

function iGuild:OnInitialize()
	self.db = LibStub("AceDB-3.0"):New("iGuildDB", self:CreateDB(), "Default").profile;

	self:GetDisplayedColumns();

	self:RegisterEvent("GUILD_MOTD", "EventHandler");
	self:RegisterEvent("PLAYER_GUILD_UPDATE", "EventHandler");
end

function iGuild:OnEnable()
	if( IsInGuild() ) then
		self:RegisterEvent("GUILD_ROSTER_UPDATE", "EventHandler");
		GuildRoster();
		RosterTimer = self:ScheduleRepeatingTimer(GuildRoster, 15);
	else
		self.Feed.text = L["No guild"];
	end
end

----------------------
-- EventHandler
----------------------

function iGuild:EventHandler(event)
	if( IsInGuild() ) then
		if( not RosterTimer ) then
			self:RegisterEvent("GUILD_ROSTER_UPDATE", "EventHandler");
			GuildRoster();
			RosterTimer = self:ScheduleRepeatingTimer(GuildRoster, 15);
		end
		
		local total, totalOn = GetNumGuildMembers();
		self.Feed.text = ("%d/%d"):format(totalOn, total);
		
		self:SetupGuildRoster();
	else
		if( RosterTimer ) then
			self:UnregisterEvent("GUILD_ROSTER_UPDATE");
			self:CancelTimer(RosterTimer);
		end
		
		self.GuildRoster = {};
		self.Feed.text = L["No guild"];
	end
	
	if( QTip:IsAcquired("iSuite"..AddonName) ) then
		self:UpdateTooltip();
	end
end

--------------------------
-- SetupGuildRoster
--------------------------

function iGuild:SetupGuildRoster()
	local _, totalOn = GetNumGuildMembers();
	
	tclear(Roster);
	
	for i = 1, totalOn do
		local charName, guildRank, _, charLevel, charClass, charZone, 
			guildNote, officerNote, isOnline, charStatus, _, acmPoints, acmRank, charMobile = GetGuildRosterInfo(i);
		
		if( isOnline ) then
			Roster[i] = {};
			Roster[i][ROSTER_CHAR_NAME] = charName;
			Roster[i][ROSTER_CHAR_LEVEL] = charLevel;
			Roster[i][ROSTER_CHAR_CLASS] = charClass;
			Roster[i][ROSTER_CHAR_ZONE] = charZone;
			Roster[i][ROSTER_CHAR_STATUS] = charStatus or nil;
			Roster[i][ROSTER_CHAR_MOBILE] = charMobile or false;
			Roster[i][ROSTER_ACM_POINTS] = acmPoints;
			Roster[i][ROSTER_ACM_RANK] = acmRank;
			Roster[i][ROSTER_GUILD_RANK] = guildRank;
			Roster[i][ROSTER_GUILD_NOTE] = guildNote or nil;
			Roster[i][ROSTER_OFFICER_NOTE] = officerNote or nil;
		end
	end
	
	table.sort(Roster, self.Sort[self.db.Sort]);
end

-----------------------
-- UpdateTooltip
-----------------------

local function iGuildLineClick(_, name, button)
	if( IsAltKeyDown() ) then
		InviteUnit(name);
	else
		SetItemRef(("player:%s"):format(name), ("|Hplayer:%s|h[%s]|h"):format(name, name), "LeftButton");
	end
end

function iGuild:UpdateTooltip()
	Tooltip:Clear();
	Tooltip:SetColumnLayout(#DisplayedColumns);
	
	local name, info;

	for y = 0, #Roster do
		local member;
		
		for x = 1, #DisplayedColumns do
			name = DisplayedColumns[x];
			info = self.Columns[name];
			
			if( x == 1 ) then
				if( y == 0) then
					Tooltip:AddHeader(" ");
				else
					Tooltip:AddLine(" ");
				end
			end
			
			if( y == 0 ) then
				if( self.db.Column[name].ShowLabel ) then
					Tooltip:SetCell(y +1, x, info.Label, nil, self.db.Column[name].Align);
				end
				
			else
				member = Roster[y];
				Tooltip:SetCell(y +1, x, info.Brush(member), nil, self.db.Column[name].Align);
				
				if( info.Script and info.ScriptUse(member) ) then
					Tooltip:SetCellScript(y +1, x, "OnMouseDown", info.Script, member);
				end
			end
		end
		
		if( member ) then
			Tooltip:SetLineScript(y +1, "OnMouseDown", iGuildLineClick, member[ROSTER_CHAR_NAME]);
		end
	end
end